<?xml version="1.0" encoding="UTF-8"?>
<!-- $Id: simple2repository.xsl 52 2007-05-21 08:39:40Z hnetynka $ -->

<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:org.objectweb.dsrg.sofa.repository.model="http:///org.objectweb.dsrg.sofa.repository.model">

	<xsl:strip-space elements="*" />
	<!-- <xsl:preserve-space elements=""/> -->
	<xsl:output method="xml" indent="yes" encoding="utf-8" />


	<!-- ================================================================================================================= -->
	<!-- Root elemenent                                                                                                    -->
	<!-- ================================================================================================================= -->
	<xsl:template match="/">
		<xsl:apply-templates />
	</xsl:template>


	<!-- ================================================================================================================= -->
	<!-- InterfaceType                                                                                                     -->
	<!-- ================================================================================================================= -->
	<xsl:template match="/itf-type">
		<org.objectweb.dsrg.sofa.repository.model:InterfaceType>
			<xsl:copy-of select="@*" />
		</org.objectweb.dsrg.sofa.repository.model:InterfaceType>
	</xsl:template>


	<!-- ================================================================================================================= -->
	<!-- Frame                                                                                                             -->
	<!-- ================================================================================================================= -->
	<xsl:template match="/frame">
		<org.objectweb.dsrg.sofa.repository.model:Frame>
			<!--<xsl:attribute name="protocol">
				<xsl:value-of select="protocol/text()" />
			</xsl:attribute> -->
			<xsl:copy-of select="@name" />

			<xsl:if test="count(@top-level) = 1">
				<annotation xsi:type="org.objectweb.dsrg.sofa.repository.model:TopLevel" />
			</xsl:if>

			<xsl:apply-templates />
		</org.objectweb.dsrg.sofa.repository.model:Frame>
	</xsl:template>

	<xsl:template match="/frame/behavior">
		<behavior>
                        <xsl:attribute name="name">
				<xsl:value-of select="@type" />
			</xsl:attribute>
                        <xsl:attribute name="value">
				<xsl:value-of select="text()" />
			</xsl:attribute>
                </behavior>
	</xsl:template>

	<xsl:template match="/frame/property">
		<property>
			<xsl:copy-of select="@name" />
			<xsl:copy-of select="@type" />
		</property>
	</xsl:template>

	<xsl:template match="/frame/provides">
		<providedInterface>
			<xsl:call-template name="handleProvidesRequires" />
		</providedInterface>
	</xsl:template>

	<xsl:template match="/frame/requires">
		<requiredInterface>
			<xsl:call-template name="handleProvidesRequires" />
		</requiredInterface>
	</xsl:template>

	<xsl:template name="handleProvidesRequires">
		<xsl:copy-of select="@name" />
		<xsl:attribute name="communicationStyle">
			<xsl:choose>
				<xsl:when test="count(@comm-style) = 1">
					<xsl:value-of select="@comm-style" />
				</xsl:when>
				<xsl:otherwise>method-invocation</xsl:otherwise>
			</xsl:choose>
		</xsl:attribute>

		<!-- TODO; Type (normal, utility) -->
		<xsl:attribute name="connectionType"><!-- TODO: This is untested -->
			<xsl:choose>
				<xsl:when test="count(@type) = 1">
					<xsl:value-of select="@type" />
				</xsl:when>
				<xsl:otherwise>normal</xsl:otherwise>
			</xsl:choose>
		</xsl:attribute>

		<interfaceType>
			<xsl:attribute name="href"><!-- TODO: Fix the type -->
				<xsl:value-of select="@itf-type" />
			</xsl:attribute>
		</interfaceType>

		<xsl:apply-templates />
	</xsl:template>

	<xsl:template match="/frame/*[name()='requires' or name()='provides']/comm-feature">
		<communicationFeature>
			<xsl:copy-of select="@name" />
			<xsl:copy-of select="@value" />
		</communicationFeature>
	</xsl:template>

	<xsl:template name="getInterfaceRef">
		<!-- The reference of an interface is formed as '//@requiredInterface.' + index of the subcomponent -->
		<xsl:param name="interfaceName" />
		<xsl:choose>
			<xsl:when test="count(/frame/requires[@name=$interfaceName]) > 0">
				<xsl:text>//@requiredInterface.</xsl:text>
				<xsl:value-of select="count(/frame/requires[@name=$interfaceName]/preceding-sibling::requires)" />
			</xsl:when>

			<xsl:when test="count(/frame/provides[@name=$interfaceName]) > 0">
				<xsl:text>//@providedInterface.</xsl:text>
				<xsl:value-of select="count(/frame/provides[@name=$interfaceName]/preceding-sibling::provides)" />
			</xsl:when>
		</xsl:choose>
	</xsl:template>

	<xsl:template match="/frame/*[name()='requires' or name()='provides']/factory">
		<annotation xsi:type="org.objectweb.dsrg.sofa.repository.model:Factory">
			<xsl:copy-of select="@method" />

			<xsl:if test="count(@return-itf) = 1">
				<xsl:attribute name="returnInterface">
					<xsl:call-template name="getInterfaceRef">
						<xsl:with-param name="interfaceName">
							<xsl:value-of select="@return-itf" />
						</xsl:with-param>
					</xsl:call-template>
				</xsl:attribute>
			</xsl:if>

                        <xsl:if test="count(@return-instance) = 1">
				<xsl:attribute name="returnInstance">
					<xsl:value-of select="@return-instance" />
				</xsl:attribute>
			</xsl:if>
		</annotation>
	</xsl:template>


	<!-- ================================================================================================================= -->
	<!-- Architecture                                                                                                      -->
	<!-- ================================================================================================================= -->

	<xsl:template match="/architecture">
		<org.objectweb.dsrg.sofa.repository.model:Architecture>
			<xsl:copy-of select="@name" />
                        <xsl:attribute name="implementation">
                                <xsl:value-of select="@impl" />
                        </xsl:attribute>

			<implements>
				<xsl:attribute name="href"><!-- TODO: Fix the type -->
					<xsl:value-of select="@frame" />
				</xsl:attribute>
			</implements>

			<xsl:apply-templates />
		</org.objectweb.dsrg.sofa.repository.model:Architecture>
	</xsl:template>

	<xsl:template name="getSubCompRef">
		<!-- The reference of a subcomponent is formed by '//@subcomponent.' + index of the subcomponent -->
		<xsl:param name="subCompName" />
		<xsl:text>//@subcomponent.</xsl:text>
		<xsl:value-of select="count(/architecture/sub-comp[@name=$subCompName]/preceding-sibling::sub-comp)" />
	</xsl:template>

	<xsl:template match="/architecture/sub-comp">
		<subcomponent>
			<xsl:copy-of select="@name" />

			<instantiatesFrame>
				<xsl:attribute name="href"><!-- TODO: Fix the type -->
					<xsl:value-of select="@frame" />
				</xsl:attribute>
			</instantiatesFrame>

			<xsl:if test="count(@arch) = 1">
				<instantiatesArchitecture>
					<xsl:attribute name="href"><!-- TODO: Fix the type -->
						<xsl:value-of select="@arch" />
					</xsl:attribute>
				</instantiatesArchitecture>
			</xsl:if>
		</subcomponent>
	</xsl:template>

	<xsl:template match="/architecture/connection">
		<connection>
			<xsl:apply-templates />
		</connection>
	</xsl:template>

	<xsl:template match="/architecture/connection/endpoint[count(@sub-comp) = 1]">
		<endpoint xsi:type="org.objectweb.dsrg.sofa.repository.model:SubcomponentInterfaceEndpoint">
			<xsl:attribute name="interfaceName">
				<xsl:value-of select="@itf" />
			</xsl:attribute>

			<xsl:attribute name="subcomponent">
				<xsl:call-template name="getSubCompRef">
					<xsl:with-param name="subCompName">
						<xsl:value-of select="@sub-comp" />
					</xsl:with-param>
				</xsl:call-template>
			</xsl:attribute>
		</endpoint>
	</xsl:template>

	<xsl:template match="/architecture/connection/endpoint[count(@sub-comp) = 0]">
		<endpoint xsi:type="org.objectweb.dsrg.sofa.repository.model:ComponentInterfaceEndpoint">
			<xsl:attribute name="interfaceName">
				<xsl:value-of select="@itf" />
			</xsl:attribute>
		</endpoint>
	</xsl:template>

	<xsl:template match="/architecture/mapped-property">
		<mappedProperty>
			<xsl:copy-of select="@name" />
			<xsl:copy-of select="@type" />

			<xsl:attribute name="instance">
				<xsl:call-template name="getSubCompRef">
					<xsl:with-param name="subCompName">
						<xsl:value-of select="@sub-comp" />
					</xsl:with-param>
				</xsl:call-template>
			</xsl:attribute>

			<xsl:attribute name="subcomponentPropertyName">
				<xsl:value-of select="@sub-comp-property" />
			</xsl:attribute>
		</mappedProperty>
	</xsl:template>

	<xsl:template match="/architecture/property">
		<property>
			<xsl:copy-of select="@name" />
			<xsl:copy-of select="@type" />
		</property>
	</xsl:template>

	<!-- ================================================================================================================= -->
	<!-- Assembly                                                                                                          -->
	<!-- ================================================================================================================= -->

        <xsl:template match="/assembly">
		<org.objectweb.dsrg.sofa.repository.model:AssemblyDescriptor>
			<xsl:copy-of select="@name" />

			<topLevelInstance>
                                <architecture>
				        <xsl:attribute name="href">
					        <xsl:value-of select="@top-level-arch" />
        				</xsl:attribute>
                                </architecture>
        			<xsl:apply-templates />
			</topLevelInstance>

		</org.objectweb.dsrg.sofa.repository.model:AssemblyDescriptor>
	</xsl:template>

        <xsl:template match="/assembly//subcomponent">
                <subcomponent>
			<xsl:copy-of select="@name" />
                        <instance>
                                <architecture>
				        <xsl:attribute name="href">
					        <xsl:value-of select="@arch" />
        				</xsl:attribute>
                                </architecture>
        			<xsl:apply-templates />
                        </instance>
                </subcomponent>
        </xsl:template>

        <xsl:template match="/assembly//dynamic-instance">
                <dynamicInstance>
                        <architecture>
        		        <xsl:attribute name="href">
				        <xsl:value-of select="@arch" />
        			</xsl:attribute>
                        </architecture>
        		<xsl:apply-templates />
                </dynamicInstance>
        </xsl:template>

	<!-- ================================================================================================================= -->
	<!-- Error reporting                                                                                                   -->
	<!-- ================================================================================================================= -->
	<xsl:template match="*|/" priority="-1">
		<xsl:message terminate="yes">
			Unexpected element found:
			<xsl:for-each select="ancestor-or-self::*">
				<xsl:value-of select="name(.)" />
				/
			</xsl:for-each>
		</xsl:message>
	</xsl:template>

	<xsl:template match="text()" priority="-1">
		<xsl:message terminate="yes">
			Unexpected text node: '
			<xsl:copy-of select="." />
			' Path is
			<xsl:for-each select="ancestor-or-self::*">
				<xsl:value-of select="name(.)" />
				/
			</xsl:for-each>
		</xsl:message>
	</xsl:template>

	<!--
		<xsl:template match="@*" priority="-1">
		<xsl:message terminate="yes">
		Unexpected attribute node: '<xsl:copy-of select="name(.)"/>'
		Path is <xsl:for-each select="ancestor-or-self::*"><xsl:value-of select="name(.)"/> / </xsl:for-each>
		</xsl:message>
		</xsl:template>
	-->

</xsl:stylesheet>
